"""
example_template.py - This is a template for defining new examples.  It is not intended to be used directly.

<describe what this example does>

In this example, we:
  - <key step 1>
  - <key step 2
  - ...

<describe assumptions about the user's stripe account, environment, or configuration;
or things to watch out for when running>
"""

import stripe

# Set your API key here
api_key = "{{API_KEY}}"

print("Hello world")
# client = stripe.StripeClient(api_key)
# client.v2....
